unit Database;

{
-- Running Chronometer library --
   Database unit
   Copyright Pierre DELORE
   Version 1.0 - 23/05/2004

Feel free to use or modify this file
For any question, improvement...
send me an email at dpliste@free.fr
}

Interface
  uses sysutils, Tools, Dialogs;

Type
  {-- Record structure --}
  TDistRec = record
    Sec : Byte;
    Min : Byte;
    Hour : Byte;
    Distance : Byte;
  End;

  {-- Database structure --}
  TBuffer = packed Record
    AllocationSize : Word;        {+0}
    DatabaseSize : Word;          {+2}
    AppSpecificHeaderSize : Byte; {+4}
    NumberOfRecord : Word;        {+5}
    RecordSize : Byte;            {+7}
    StartTimeSec    : Byte;       {+8}
    StartTimeMin    : Byte;       {+9}
    StartTimeHour   : Byte;       {+10}
    StartDateDay    : Byte;       {+11}
    StartDateMonth  : Byte;       {+12}
    StartDateYearLo : Byte;       {+13}
    StartDateYearHi : Byte;       {+14}
    TargetDist      : Byte;       {+15}
    NumRecUsed      : Byte;       {+16}
    TargetDist10    : Byte;       {+17}
    ID1             : Byte;       {+18}
    ID2             : Byte;       {+19}
    ID3             : Byte;       {+20}
    DistRec : Array [1..99] of TDistRec; {+21}
  end;


  procedure WriteCSV(sBaseDir:String);
  procedure WriteDB(sBaseDir:String);
  procedure WriteDBex(sBaseDir:String);
  function ReadDB(sName:String):Boolean;
  function SearchDB(sBaseDir:String; sID:String; p:Integer):String;

Var
  Buffer, cBuffer : TBuffer;
  BaseDir : String;


Implementation


{-- Write the database in VCS format --}
procedure WriteCSV(sBaseDir:String);
Var
  F:TextFile;
  i:integer;
  h,m,s,d:byte;
  str:String;
begin
  {- Create the RChrono directory if it doesn't exist -}
  if not DirectoryExists(sBaseDir+'RChrono') then
    if not CreateDir(sBaseDir+'RChrono') then
      raise Exception.Create('Cannot create '+sBaseDir+'RChrono');

  {$I-}
  With Buffer do
  Begin
    str:=IntToStr(Bcd2Word(StartDateYearHi,StartDateYearLo))+
            FormatByte(Bcd2Byte(Buffer.StartDateMonth))+
            FormatByte(Bcd2Byte(Buffer.StartDateDay))+'-'+
            FormatByte(Bcd2Byte(Buffer.StartTimeHour))+
            FormatByte(Bcd2Byte(Buffer.StartTimeMin))+
            FormatByte(Bcd2Byte(Buffer.StartTimeSec))+
            '.csv';

    {$I-}
    AssignFile(F, sBaseDir+'RChrono\'+str);
    Rewrite(F);
    WriteLn(F,'Year,'+IntToStr(Bcd2Word(StartDateYearHi,StartDateYearLo)));
    WriteLn(F,'Month,'+FormatByte(Bcd2Byte(Buffer.StartDateMonth)));
    WriteLn(F,'Day,'+FormatByte(Bcd2Byte(Buffer.StartDateDay)));
    WriteLn(F,'Hour,'+FormatByte(Bcd2Byte(Buffer.StartTimeHour)));
    WriteLn(F,'Min,'+FormatByte(Bcd2Byte(Buffer.StartTimeMin)));
    WriteLn(F,'Sec,'+FormatByte(Bcd2Byte(Buffer.StartTimeSec)));
    WriteLn(F,'Target distance,'+intToStr(TargetDist)+'.'+intToStr(TargetDist10));
  End;
  h:=0;
  m:=0;
  s:=0;
  d:=0;
  WriteLn(F,'N,Split Time,Distance,Lap Time,Target Time,Average speed');
  for i:=1 to Buffer.NumRecUsed do
  Begin
    With Buffer.DistRec[i]do
    Begin
      Writeln(F,
        intToStr(i)+','+
        FormatTime(Hour,Min,Sec)+','+
        intToStr(Distance)+','+
        LapTime(h,m,s,Hour,Min,Sec)+','+
        TargetTime(Hour,Min,Sec,Distance,Buffer.TargetDist)+','+
        AverageSpeed(h,m,s,Hour,Min,Sec,Distance-d));
      h:=Hour;
      m:=Min;
      s:=Sec;
      d:=Distance;
    End;
  End;
  WriteLn(F,'Total Average Speed,'+AverageSpeed(0,0,0,h,m,s,d));
  CloseFile(F);
  {$I+}
  If IOResult<>0 then ShowMessage('Cannot create '+str);
end;


{-- Write the database --}
procedure WriteDB(sBaseDir:String);
Var
  F:File of byte;
  i:integer;
  b:byte;
begin
  With Buffer do
  begin
   DatabaseSize:=SizeOf(Buffer)-2;
   AllocationSize:=(((DatabaseSize-1) div 64)+1)*64;
   AppSpecificHeaderSize:=16;
   NumberOfRecord:=99;
   RecordSize:=4;
   ID1:=Ord('R');
   ID2:=Ord('C');
   ID3:=Ord('H');
  end;
  For i:=1 to 99 do
  With Buffer.DistRec[1] do
  begin
    Sec:=0;
    Min:=0;
    Hour:=0;
    Distance:=0;
  end;

  {$I-}
  AssignFile(F, sBaseDir+'App\rchrono_dbase_018.bin');
  Rewrite(F);
  B:=LO(Buffer.AllocationSize);
  Write(F,B);
  B:=HI(Buffer.AllocationSize);
  Write(F,B);

  B:=LO(Buffer.DatabaseSize);
  Write(F,B);
  B:=HI(Buffer.DatabaseSize);
  Write(F,B);

  Write(F,Buffer.AppSpecificHeaderSize);
  B:=LO(Buffer.NumberOfRecord);
  Write(F,B);
  B:=HI(Buffer.NumberOfRecord);
  Write(F,B);

  Write(F,Buffer.RecordSize);
  Write(F,Buffer.StartTimeSec);
  Write(F,Buffer.StartTimeMin);
  Write(F,Buffer.StartTimeHour);
  Write(F,Buffer.StartDateDay);
  Write(F,Buffer.StartDateMonth);
  Write(F,Buffer.StartDateYearLo);
  Write(F,Buffer.StartDateYearHi);
  Write(F,Buffer.TargetDist);
  Write(F,Buffer.NumRecUsed);
  Write(F,Buffer.TargetDist10);
  Write(F,Buffer.ID1);
  Write(F,Buffer.ID2);
  Write(F,Buffer.ID3);

  For i:=1 to 99 do
    With Buffer.DistRec[i] do
      Write(F,Sec,Min,Hour,Distance);

  CloseFile(F);
  {$I+}
  If IOResult<>0 then ShowMessage('Cannot create "rchrono_dbase_018.bin"');
end;


{-- Write the database with existing datas --}
procedure WriteDBex(sBaseDir:String);
Var
  F:File of byte;
  i:integer;
  b:byte;
begin
  With Buffer do
  begin
   DatabaseSize:=SizeOf(Buffer)-2;
   AllocationSize:=(((DatabaseSize-1) div 64)+1)*64;
   AppSpecificHeaderSize:=17;
   NumberOfRecord:=99;
   RecordSize:=4;
   ID1:=Ord('R');
   ID2:=Ord('C');
   ID3:=Ord('H');
  end;
  {
  For i:=1 to 99 do
  With Buffer.DistRec[1] do
  begin
    Sec:=0;
    Min:=0;
    Hour:=0;
    Distance:=0;
  end;
  }

  {$I-}
  AssignFile(F, sBaseDir+'App\rchrono_dbase_018.bin');
  Rewrite(F);
  B:=LO(Buffer.AllocationSize);
  Write(F,B);
  B:=HI(Buffer.AllocationSize);
  Write(F,B);

  B:=LO(Buffer.DatabaseSize);
  Write(F,B);
  B:=HI(Buffer.DatabaseSize);
  Write(F,B);

  Write(F,Buffer.AppSpecificHeaderSize);
  B:=LO(Buffer.NumberOfRecord);
  Write(F,B);
  B:=HI(Buffer.NumberOfRecord);
  Write(F,B);

  Write(F,Buffer.RecordSize);
  Write(F,Buffer.StartTimeSec);
  Write(F,Buffer.StartTimeMin);
  Write(F,Buffer.StartTimeHour);
  Write(F,Buffer.StartDateDay);
  Write(F,Buffer.StartDateMonth);
  Write(F,Buffer.StartDateYearLo);
  Write(F,Buffer.StartDateYearHi);
  Write(F,Buffer.TargetDist);
  Write(F,Buffer.NumRecUsed);
  Write(F,Buffer.TargetDist10);
  Write(F,Buffer.ID1);
  Write(F,Buffer.ID2);
  Write(F,Buffer.ID3);

  For i:=1 to 99 do
    With Buffer.DistRec[i] do
      Write(F,Sec,Min,Hour,Distance);

  CloseFile(F);
  {$I+}
  If IOResult<>0 then ShowMessage('Cannot create "rchrono_dbase_018.bin"');
end;


{-- Read the database in the buffer --}
function ReadDB(sName:String):Boolean;
Var
  F:File of byte;
  i:integer;
begin
  For i:=1 to 99 do
  With Buffer.DistRec[1] do
  begin
    Sec:=0;
    Min:=0;
    Hour:=0;
    Distance:=0;
  end;

  {$I-}
  AssignFile(F, sName);
  Reset(F);
  Seek(F,8);
  With Buffer do
  Begin
    Read(F,
          StartTimeSec,
          StartTimeMin,
          StartTimeHour,
          StartDateDay,
          StartDateMonth,
          StartDateYearLo,
          StartDateYearHi,
          TargetDist,
          NumRecUsed,
          TargetDist10
        );
  End;


  Seek(F,21);
  For i:=1 to Buffer.NumRecUsed do
    With Buffer.DistRec[i] do
      Read(F,Sec,Min,Hour,Distance);

  CloseFile(F);
  {$I+}
  cBuffer:=Buffer;
  {-- Convert Buffer form bcd to dec (only records)--}
  For i:=1 to Buffer.NumRecUsed do
    With Buffer.DistRec[i] do
    Begin
      Sec:=Bcd2Byte(Sec);
      Min:=Bcd2Byte(Min);
      Hour:=Bcd2Byte(Hour);
    End;

  if IOResult=0 then ReadDB:=True
  else ReadDB:=False;
end;


{-- Search the database in the DB dir --}
{
 Search if the database exist:
 1/ Search for files 'upwristappdb*.bin' in the 'DB' directory
 2/ Then look if we find the ID 'RCH' in the file at position p
}
function SearchDB(sBaseDir:String; sID:String; p:Integer):String;
Var
  s,sn  : String;
  sr    : TSearchRec;
  Found : Boolean;
  F     : File of byte;
  B     : Byte;
begin
  s:='';
  sn:='';
  Found:=False;
  //ShowMessage('Debut recherche');  Debug
  if FindFirst(sBaseDir+'DB\upwristappdb*.bin', faAnyFile , sr) = 0  then
    begin
      repeat
        {$I-}
        AssignFile(F, sBaseDir+'DB\'+sr.Name);
        Reset(F);
        Seek(F,p);
        read(F,B);
        s:=s+chr(B);
        read(F,B);
        s:=s+chr(B);
        read(F,B);
        s:=s+chr(B);
        CloseFile(F);
        {$I+}
        if s=sID then
          begin
            Found:=True;
            sn:=sr.Name;
          end;
        //ShowMessage(sr.Name+' = '+s); Debug
      until FindNext(sr) <> 0;
      FindClose(sr);
    end;
  If Found then SearchDB:=sBaseDir+'DB\'+sn
  else SearchDB:='';
end;

end.
