;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name     : rchBckHd.asm
; Purpose       : Handles the following application specific functions:
;                   - application initialization
;                   - resource refresh
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchBckHd'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchBackgroundHandler
; Description : Handles application initialization and refresh resource handlers.
; Assumptions : COREInitializationASDAddress is already set by kernel.
;               COREInitializationADDAddress is already set by kernel.
; Input(s)    : None
; Output(s)   : None
;               ( Destroyed: All registers )
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchBackgroundHandler

rchBackgroundHandler:

                ; Load the event to be process to AReg.
                ld      A, [COREBackgroundEvent]

                ; Check if INIT event.
                cp      A, #COREEVENT_INIT
                jr      Z, rchBackgroundInitEvent

                ; Check if TASK EXIT event.
                cp      A, #COREEVENT_TASKEXIT
                jr      Z, rchBackgroundTaskExit

                ret
                
;--------------------------------------------------------------------
rchBackgroundInitEvent:

                ;TODO: Initialize your variables here
                ;Called after a sync
                
                ld      IY, [COREInitializationASDAddress]

                ld      B, #(RCHSTATUS-RCHSPLIT_HU)
                ld      HL,IY
                add     HL,#RCHSPLIT_HU
                KRES_CLEAR_N_BYTES
                
                ld      A,#01
                ld      [IY + RCHDISTOFFSET], A
                ld      [IY + RCHTARGETOFFSET], A
                ld      [IY + RCHDRINKV],A
                ld      [IY + RCHEATV],A
                ld      [IY + RCHSTATUS],A
                ld      [IY + RCHAUTODISTV],A
                dec     A
                ld      [IY + RCHPREFDISP], A
                
                ld      A, [COREBackgroundAppIndex]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE  CHR_RSRC_FLAG, FLAG_OFF
                
                ret
                
;--------------------------------------------------------------------
rchBackgroundTaskExit:

                ld      IY, [COREInitializationASDAddress]
                ld      HL,IY
                add     HL,#RCHSTATUS
                or      [HL],#02


                ret