;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchDTop.asm
; Purpose      : RChrono Application Top Display State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchdtop'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchDTopStateManager
; Description : RChrono Application Top Display State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchDTopStateManager

rchDTopStateManager:
                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchDTopStateStateEntryEvent

                ; Check if mode depress event.
                cp      A, #RCH_MODEDEPRESS
                jr      Z, rchDTopStateModeDepressEvent

                ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
rchDTopStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************

                ld      HL,[RCHTmpStatus]
                and     [HL],#0FCh

                ;-- Display No DB --
                ld      HL,[RCHTmpStatus]
                bit     [HL],#bRCHNoDbMsg
                jr      Z,next0
                    car rchTopDisplayNoDb
                    jr  EndState
                
next0:          ;Display EAT
                bit     [HL],#bRCHEatMsg
                jr      Z,next1
                    car rchTopDisplayEat
                    jr  EndState
                
next1:          ;Display DRINK
                bit     [HL],#bRCHDrinkMsg
                jr      Z,next2
                    car rchTopDisplayDrink
                    jr  EndState
                
next2:          ld      A,[IY+RCHDISPLAYTOP]
                
                ; Display Time
                cp      A,#00
                jr      NZ,next3
                    car rchTopDisplayTime
                    jr  EndState
                    
                ; Display Target
next3:          cp      A,#01
                jr      NZ,next4
                    car rchTopDisplayTarget
                    jr EndState
                    
                ; Display Split
next4:          cp      A,#02
                jr      NZ,next5
                    car RchTopDisplaySplit
                    jr  EndState
                    
                ; Display LAP
next5:          cp      A,#03
                jr      NZ,next6
                    car RchTopDisplayLap
                    jr  EndState

next6:
EndState:       
                ld      B, #RCHDEFAULTSTATE
                CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY                    
                ret
                 
;------------------------------------------------------------------------------
rchDTopStateModeDepressEvent:
                ;**************************************************************
                ;
                ;                       MODE DEPRESS
                ;
                ;**************************************************************
                ;This is default to go to the next mode
                CORE_REQ_MODE_CHANGE_NEXT
                ret


;**************************************************************
;
;  Display Time in the segment area
;
;**************************************************************

rchLocalTimeBuffer:
rchMin:         db 00 ;minute
rchHour:        db 00 ;hour
                db 00 ;date
                db 00 ;month
                db 00 ;year lo
                db 00 ;year hi
                db 00 ;dow
;***************************************
rchTopDisplayTime:

                ;now, copy hour into our local buffer
                ld  A, [COREPTZIndex]
                ld  IY, #rchLocalTimeBuffer
                KTOD_COPY_TIME_MIN_TO_DOW_FROM_RESOURCE

                ;Clear the 9 segments line
                LCD_CLR_SEG_LINE

                ; Display Hour
                ld A, [rchHour]
                ld IX, #LCDSEGDIGIT3
                LCD_DISP_2DIG_SEG_DATA_SUP_ZERO_MSD

                ; Display Minute
                ld A, [rchMin]
                ld IX, #LCDSEGDIGIT5
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP

                ; Turn on the COLON between Segmented Digits 4 and 5
                LCD_UPD_COLON1_FLAG OFF
                LCD_UPD_COLON2_FLAG ON
                
                ld  IY,#RchTimePattern
                car RchDisplayPattern
                
                ret


;**************************************************************
;
;  Display the target time in the segment area
;
;**************************************************************
rchTopDisplayTarget:
                ld      IY, [CORECurrentASDAddress]
                add     IY,#RCHTARGET_S
                car RchDispHMSSeg
                
                ld  IY,#RchTargetPattern
                car RchDisplayPattern

                ret


;**************************************************************
;
;  Display the split time in the segment area
;
;**************************************************************
RchTopDisplaySplit:
                ld      IY, [CORECurrentASDAddress]
                add     IY,#RCHSPLIT_S
                car RchDispHMSSeg
                
                ;-- Display split number --
                ld      IY, [CORECurrentASDAddress]
                ld      L, [IY+RCHDISTOFFSET]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD

                ;Display RCHDISTOFFSET on upper display
                ld      IX, #LCDUPPERDMCOL1
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ret

                
;**************************************************************
;
;  Display the lap time in the segment area
;
;**************************************************************
RchTopDisplayLap:
                ld      IY, [CORECurrentASDAddress]
                add     IY,#RCHLAP_S
                car RchDispHMSSeg

                ld  IY,#RchLapPattern
                car RchDisplayPattern

                ret

                
;**************************************************************
;
;  Display the HMS in the segment area
;
;**************************************************************
; IY = Address of the structure to display
; Structure:
; Second/Minute/Hour

RchDispHMSSeg:                
                ; Turn on the COLON between Segmented Digits 4 and 5
                LCD_UPD_COLON1_FLAG ON
                LCD_UPD_COLON2_FLAG ON
                
                ; Display Hour
                ld A, [IY+2]
                ld IX, #LCDSEGDIGIT1
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ; Display Minute
                ld A, [IY+1]
                ld IX, #LCDSEGDIGIT3
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ; Display Sec
                ld A, [IY]
                ld IX, #LCDSEGDIGIT5
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ret

;**************************************************************
;
;  Display EAT and logo in the segment area
;
;**************************************************************
rchTopDisplayEat:
                ld  HL,[RCHTmpStatus]
                and [HL],#(~bRCHEatMsg)

                ld  IY, #MsgEat
                jr  LDispMsgED
rchTopDisplayDrink:   
                ld  HL,[RCHTmpStatus]
                and [HL],#(~bRCHDrinkMsg)

                ld  IY, #MsgDrink
LDispMsgED:     LCD_DISP_SEG_LINE_MSG
                
                LCD_UPD_COLON1_FLAG OFF
                LCD_UPD_COLON2_FLAG OFF

                ld  IY,#RchInfoPattern
                car RchDisplayPattern
                
                ret

;**************************************************************
;
;  Display NO DB in the segment area
;
;**************************************************************
rchTopDisplayNoDb:
                ld  IY, #MsgNoDb
                LCD_DISP_SEG_LINE_MSG
                
                LCD_UPD_COLON1_FLAG OFF
                LCD_UPD_COLON2_FLAG OFF

                ld  IY,#RchInfoPattern
                car RchDisplayPattern
                
                ret

                
                
;-- Display the pattern --
; IY = Address of the 11 pattern
;
RchDisplayPattern:
                ld B, #11
                ld IX, #0F802H
                UTL_COPY_IYREG_TO_IXREG
                ret
            
MsgEat:         db SEG_SPACE, SEG_E, SEG_A, SEG_T, SEG_SPACE, SEG_SPACE 
MsgDrink:       db SEG_SPACE, SEG_D, SEG_R, SEG_I, SEG_N, SEG_K
MsgNoDb:        db SEG_SPACE, SEG_N, SEG_O, SEG_SPACE, SEG_D, SEG_B 


RchTimePattern:     db 00H,00H,0EH,11H,11H,17H,15H,15H,0EH,00H,00H
RchTargetPattern:   db 04H,04H,04H,04H,1FH,0EH,0EH,15H,15H,11H,0EH
RchLapPattern:      db 1FH,04H,04H,04H,04H,04H,1FH,0EH,0EH,04H,1FH
RchInfoPattern:     db 00H,00H,00H,00H,00H,10H,1DH,1DH,10H,00H,17H

