;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchResume.asm
; Purpose      : RChrono Application Resume Display State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchresume'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchResumeStateManager
; Description : RChrono Application Top Display State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchResumeStateManager

rchResumeStateManager:
                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchResumeStateStateEntryEvent

                ; Check if mode depress event.
                cp      A, #RCH_MODEDEPRESS
                jr      Z, rchResumeStateModeDepressEvent

                ; Check if start/split depress event.
                cp      A, #RCH_STARTSPLITDEPRESS
                jr      Z, rchResumeStateStartSplitDepressEvent

                ; Check if stop/reset depress event.
                cp      A, #RCH_STOPRESETDEPRESS
                jr      Z, rchResumeStateStopResetDepressEvent

                ; Check if timeout hi-res done event.
                cp      A, #RCH_TIMEOUTDONEHIGHRES
                jr      Z, rchResumeStateTimeoutHiResDoneEvent

                ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
rchResumeStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************

                ;-- Display distance in the upper DM --
                ld      L, [IY + RCHDISTOFFSET]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD
                ld IX, #LCDUPPERDMCOL1
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ;-- Display Split in the segment area --
                ld A,[IY+RCHSPLIT_H]
                ld IX, #LCDSEGDIGIT1
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ld A,[IY+RCHSPLIT_M]
                ld IX, #LCDSEGDIGIT3
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ld A,[IY+RCHSPLIT_S]
                ld IX, #LCDSEGDIGIT5
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP

                LCD_UPD_COLON1_FLAG ON
                LCD_UPD_COLON2_FLAG ON
                
                ;-- Display Target in the 1st line --
                ld A, [IY+RCHTARGET_H]
                ld IX, #LCDMAINDMLINE1COL5
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ld L, #DM5_COLON
                ld IX, #LCDMAINDMLINE1COL17
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

                ld A, [IY+RCHTARGET_M]
                ld IX, #LCDMAINDMLINE1COL19
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ld A, [IY+RCHTARGET_S]
                ld IX, #LCDMAINDMLINE1COL32
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ;-- Display Lap in the 2nd line -- 
                ld A, [IY+RCHLAP_H]
                ld IX, #LCDMAINDMLINE2COL5
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ld L, #DM5_COLON
                ld IX, #LCDMAINDMLINE2COL17
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

                ld A, [IY+RCHLAP_M]
                ld IX, #LCDMAINDMLINE2COL19
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ld A, [IY+RCHLAP_S]
                ld IX, #LCDMAINDMLINE2COL32
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ;-- Display icon --
                ld B, #3
                ld  IY,#PatternT
                ld IX, #0F902H
                UTL_COPY_IYREG_TO_IXREG

                ld B, #3
                ld  IY,#PatternL
                ld IX, #0FA02H
                UTL_COPY_IYREG_TO_IXREG
                
                ; Request 5sec timeout.
                CORE_REQ_TIMEOUT_HIRES  RCHHIRESTO_5SECONDS

                ret

PatternT:       db 02H,1EH,02H
PatternL:       db 3CH,20H,20H


;------------------------------------------------------------------------------
                ;**************************************************************
                ;
                ;                       START/SPLIT DEPRESS
                ;                       STOP/RESET DEPRESS
                ;                       TIMEOUT HIRES DONE
                ;**************************************************************
rchResumeStateTimeoutHiResDoneEvent:
rchResumeStateStartSplitDepressEvent:
rchResumeStateStopResetDepressEvent:
EndState:       
                ld      B, #RCHDEFAULTSTATE
                CORE_REQ_STATE_CHANGE                    
                ret
                 
                
;------------------------------------------------------------------------------
rchResumeStateModeDepressEvent:
                ;**************************************************************
                ;
                ;                       MODE DEPRESS
                ;
                ;**************************************************************
                ;This is default to go to the next mode
                CORE_REQ_MODE_CHANGE_NEXT
                ret

