;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name :   rchUtil.asm
; Purpose   :   RChrono Wrist App Common Utility Routines
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchutil'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchCurrentList
                GLOBAL  rchCurrentListOffset
                GLOBAL  rchTotalLists
                GLOBAL  rchBannerBuffer
                GLOBAL  rchValueBuffer
                GLOBAL  rchDatabaseBuffer
                GLOBAL  rchOpenDatabase
                GLOBAL  rchCloseDatabase
                GLOBAL  rchDBHund
                GLOBAL  rchDBDistance
                GLOBAL  rchDBHours
                GLOBAL  rchDBMin
                GLOBAL  rchDBSec
                GLOBAL  rchHTNumRecLo
                GLOBAL  rchHTNumRecHi
                GLOBAL  rchHRecSize
                GLOBAL  rchHSec
                GLOBAL  rchHMin
                GLOBAL  rchHHour
                GLOBAL  rchHDate
                GLOBAL  rchHMonth
                GLOBAL  rchHYearLo
                GLOBAL  rchHYearHi
                GLOBAL  rchHTargetDist
                GLOBAL  rchHNumRec
                GLOBAL  rchWriteInitHeader
                GLOBAL  rchWriteNbRecHeader
                GLOBAL  rchReadHeader
                GLOBAL  rchHeaderBuffer                
                GLOBAL  rchHID1


;----------------------------------------------------------------------------------
; open database.  Done at the beginning of each state handler
;----------------------------------------------------------------------------------
rchOpenDatabase:
                ; open the database and read the first record
                ld      HL, [CORECurrentADDAddress]
                ld      [DBExternalMemoryAddress], HL
                DB_OPEN_FILE
                ret


;----------------------------------------------------------------------------------             
; close the database.  Done at the end of each state handler
;----------------------------------------------------------------------------------
rchCloseDatabase:
                ; close the database and power down the eeprom
                DB_CLOSE_FILE
                ret


;----------------------------------------------------------------------------------
; Read the header into the header buffer
; uses IY - record index
;----------------------------------------------------------------------------------
rchReadHeader:
                car rchOpenDatabase
                
                ; find the record and load it   
                ; set the buffer (passed in IY)
                ld      HL, #rchHeaderBuffer
                ld      [DBInternalMemoryAddress], HL
                
                ; setup the number of bytes to write to the header information
                ld      A, #16
                ld      [DBLengthLo], A
                
                ; setup the offset from the start of the header information to begin writing
                ld      A, #0
                ld      [DBRecordOffset], A
                
                ; write the updated header information back into the database
                DB_READ_APPLICATION_INFOHEADER

                car rchCloseDatabase
                
                ;-- Test the database is exist --
                ld      A,[rchHID1]
                cp      A,#052H  ;Ascii code of R
                jr      Z,EndReadHeader
                    ld      HL,[RCHTmpStatus]
                    or      [HL],#bRCHNoDbMsg
EndReadHeader:                
                ret

                
;----------------------------------------------------------------------------------
; Write the Number of record in the header buffer
;----------------------------------------------------------------------------------
rchWriteNbRecHeader:
                ; setup the absolute address of the current application's database
                ld      IY, [CORECurrentADDAddress] ;ici
                ld      [DBExternalMemoryAddress], IY

                ; set the buffer (passed in IY)
                ld      HL, #rchHNumRec
                ld      [DBInternalMemoryAddress], HL
                
                ; setup the number of bytes to write to the header information
                ld      A, #1
                ld      [DBLengthLo], A
                
                ; setup the offset from the start of the header information to begin writing
                ld      A, #11    ;We begin to write from rchHSec
                ld      [DBRecordOffset], A
                
                ; write the updated header information back into the database
                DB_WRITE_APPLICATION_INFOHEADER
                ret
                

;----------------------------------------------------------------------------------
; Write the start time/date and target distance into the header buffer
; (Write from Sec to Target distance)
;----------------------------------------------------------------------------------
rchWriteInitHeader:
                ; copy the primary time zone data to buffer
                ; rchHTargetDist and rchHNumRec are destroy
                ld      A, [COREPTZIndex]
                ld      IY, #rchHSec
                KTOD_COPY_TIME_FROM_RESOURCE

                ; Write TargetDist
                ld      IY, [CORECurrentASDAddress]
                
                ld      A, [IY+RCHTARGETOFFSET]
                ld      [rchHTargetDist],A

                ld      A, [IY+RCHTARGET10]
                ld      [rchHTarget10],A
                
                ; Write 0 in rchHNumRec
                ld      A,#0
                ld      [rchHNumRec],A

                car     rchOpenDatabase     ;Open database

                ; setup the absolute address of the current application's database
                ld      IY, [CORECurrentADDAddress] ;ici
                ld      [DBExternalMemoryAddress], IY
                
                ; set the buffer (passed in IY)
                ld      HL, #rchHeaderBuffer
                ld      [DBInternalMemoryAddress], HL
                
                ; setup the number of bytes to write to the header information
                ld      A, #16
                ld      [DBLengthLo], A
                
                ; setup the offset from the start of the header information to begin writing
                ld      A, #0   ;We begin to write from rchHNumRec
                ld      [DBRecordOffset], A
                
                ; write the updated header information back into the database
                DB_WRITE_APPLICATION_INFOHEADER

                car     rchCloseDatabase    ;Close database
                ret


;----------------------------------------------------------------------------------
;-- Buffer for the header --            
rchHeaderBuffer:
rchHTNumRecLo:  db 00   ;+0  : Total number of record (Low)
rchHTNumRecHi:  db 00   ;+1  : Total number of record (high)
rchHRecSize:    db 00   ;+2  : Size of a record in byte
rchHSec:        db 00   ;+3  : second (BCD)
rchHMin:        db 00   ;+4  : minute (BCD)
rchHHour:       db 00   ;+5  : hour (BCD)
rchHDate:       db 00   ;+6  : date (BCD)
rchHMonth:      db 00   ;+7  : month (BCD)
rchHYearLo:     db 00   ;+8  : year lo (BCD)
rchHYearHi:     db 00   ;+9  : year hi (BCD)
rchHTargetDist: db 00   ;+10 : Target distance (DEC)
rchHNumRec:     db 00   ;+11 : Number of record used (DEC)
rchHTarget10:   db 00   ;+12 : Target distance (1/10) (DEC)
rchHID1:        db 00   ;+13 : ID N1
rchHID2:        db 00   ;+14 : ID N2
rchHID3:        db 00   ;+15 : ID N3

                
rchDBHund:      db 00   ; Hundredths (not used but necessary for display routine)

;-- Buffer for the record --
rchDatabaseBuffer:
rchDBSec:       db 00   ; secondes (BCD)
rchDBMin:       db 00   ; minutes (BCD)
rchDBHours:     db 00   ; hours (BCD)
rchDBDistance:  db 00   ; Distance (DEC)

;-- END OF FILE --
