;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchCalc.asm
; Purpose      : RChrono Application Calculation State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchcalc'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchCalcStateManager
; Description : RChrono Application Calculation State Manager.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchCalcStateManager

rchCalcStateManager:
                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchCalcStateStateEntryEvent

                ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
rchCalcStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************

                ;play sound
                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ld      HL,[RCHTmpStatus]
                or      [HL],#02

                ;Calculate the lap time
                car     rchCalcLap

                ;Calculate the target time
                car     rchCalcTarget

                ld      A,#1    
                ld      [RCHTmpDbWrite],A
                
                ;-- Goto to the database state --
                ld      B, #RCHDBSTATE
                CORE_REQ_STATE_CHANGE
                ret

                
;###################################################################################
                
Calc_hu:        db 00H ;hundredths
Calc_s:         db 00H ;second
Calc_m:         db 00H ;minute
Calc_h:         db 00H ;hour

Tmp_hu:         db 00H ;hundredths
Tmp_s:          db 00H ;second
Tmp_m:          db 00H ;minute
Tmp_h:          db 00H ;hour


;**************************************************************
;   CALC TARGET FUNCTION
;   # Take the split time and calculate the target time
;   # Result stored in RCHTARGET_..
;   Input  : None
;   Output : None 
;**************************************************************
rchCalcTarget:
                ;copy 4 bytes from RCHSPLIT structure to the Calc structure
                ld  B, #4
                ld  IY, [CORECurrentASDAddress]
                add IY, #RCHSPLIT_HU
                ld  IX, #Calc_hu
                UTL_COPY_IYREG_TO_IXREG

                ;convert split time buffer from BCD to HEX
                ld  IX, #Calc_hu
                UTL_CONVERT_4BYTE_FROM_BCD_TO_HEX

                ;Divide the split time by the done distance
                ld  IY, [CORECurrentASDAddress]
                ld  A,[IY+RCHDISTOFFSET]
                ld  IX, #(Calc_hu+3)
                UTL_DIVIDE_HMSh_BY_N_IN_HEX

                ;copy HMSh result in a buffer
                ld      B, #4
                ld      IY,#Calc_hu
                ld      IX,#Tmp_hu
                UTL_COPY_IYREG_TO_IXREG
                
                ;Multiply the time (= time by unit) by the total distance
                ld  IY, [CORECurrentASDAddress]
                ld  A,[IY+RCHTARGETOFFSET]
                ld  IY,#Calc_hu
                car UTL_MULT_HMSh_BY_N_IN_HEX

                ;convert back result to BCD
                ld  IX, #Calc_hu
                UTL_CONVERT_4BYTE_FROM_HEX_TO_BCD

                ;-- 1/10 calc --
                ld  A,#10
                ld  IX, #(Tmp_hu+3)
                UTL_DIVIDE_HMSh_BY_N_IN_HEX
                
                ld  IY, [CORECurrentASDAddress]
                ld  A,[IY+RCHTARGET10]
                ld  IY,#Tmp_hu
                car UTL_MULT_HMSh_BY_N_IN_HEX

                ;convert back result to BCD
                ld  IX, #Tmp_hu
                UTL_CONVERT_4BYTE_FROM_HEX_TO_BCD
                
                ;clear all variables in the buffer prior to loading the required
                ;data for substraction
                KRES_CLEARDATABUFFERS
                
                ;load number of hundredths to subtract
                ld      A, [Tmp_hu]
                ld      [KRESHundBuffer], A
                
                ;load number of seconds to subtract
                ld      A, [Tmp_s]
                ld      [KRESSecBuffer], A
                
                ;load number of minute to subtract
                ld      A, [Tmp_m]
                ld      [KRESMinBuffer], A
                
                ;load number of hour to subtract
                ld      A, [Tmp_h]
                ld      [KRESHrBuffer], A

                ;setup hour rollover value
                ld      A, #99H
                ld      [KRESHrRollOverValue], A
                
                ;subtract the accumulated hundredths into the structure
                ld      HL,#Calc_hu
                UTL_ADD_HMSH
                
                ;copy 4 bytes from Calc structure to the RCHTARGET structure
                ld      B, #4
                ld      IY,#Calc_hu
                ld      IX,[CORECurrentASDAddress]
                add     IX,#RCHTARGET_HU
                UTL_COPY_IYREG_TO_IXREG
                ret
                

;**************************************************************
;   CALC LAP FUNCTION
;   # Substract the a copy of the split time with the last split time
;   # Result stored in RCHLAP_..
;   Input  : None
;   Output : None 
;**************************************************************
rchCalcLap:
                ld      B, #4
                add     IY,#RCHSPLIT_HU
                ld      IX,#Split_Hu
                UTL_COPY_IYREG_TO_IXREG

                ;clear all variables in the buffer prior to loading the required
                ;data for substraction
                KRES_CLEARDATABUFFERS
                
                ;load number of hundredths to subtract
                ld      A, [RCHTmp_HU]
                ld      [KRESHundBuffer], A
                
                ;load number of seconds to subtract
                ld      A, [RCHTmp_S]
                ld      [KRESSecBuffer], A
                
                ;load number of minute to subtract
                ld      A, [RCHTmp_M]
                ld      [KRESMinBuffer], A
                
                ;load number of hour to subtract
                ld      A, [RCHTmp_H]
                ld      [KRESHrBuffer], A

                ;setup hour rollover value
                ld      A, #99H
                ld      [KRESHrRollOverValue], A
                
                ;subtract the accumulated hundredths into the structure
                ld      HL,#Split_Hu
                KRES_HUNDSECMINHR_SUB
                
                ;copy 4 bytes from Split to RCHLAP
                ld      B, #4
                ld      IY,#Split_Hu
                ld      IX, [CORECurrentASDAddress]
                add     IX, #RCHLAP_HU
                UTL_COPY_IYREG_TO_IXREG

                ret

;--------------------------------------------------------------------
;   UTL_MULT_HMSh_BY_N_IN_HEX
;
; Description :
;   Multiply a data buffer of the format Hr:Min:Sec.Hun by N and places
;   the result in the buffer.
; Usage :
;   UTL_DIVIDE_HMSH_BY_N_IN_HEX
; Assumptions :
;   Dataset is stored in a consecutive 4 byte buffer, order least significant
;   to most significant.
;   Data is unsigned, in Hexadecimal format
; Input :
;   IY - Points to the first byte of the data buffer (hundreths). (HEX data)
;   A  - Multiplicator. (HEX data)
; Output :
; Destroys : 
;--------------------------------------------------------------------
; 
UTL_Mult_Hu:    db 1
UTL_Mult_s:     db 1
UTL_Mult_m:     db 1
UTL_Mult_h:     db 1
UTL_Mult:       db 1

UTL_MULT_HMSh_BY_N_IN_HEX:
                push all
                
                ld  [UTL_Mult],A
                
                ld  B, #4
                ld  IX,#UTL_Mult_Hu
                UTL_COPY_IYREG_TO_IXREG

                ld  L,[UTL_Mult_Hu]
                ld  A,[UTL_Mult]
                mlt
                ld  A,#100
                div
                ld  [UTL_Mult_Hu],H
                ld  H,#0
                push HL

                ;Calc_s=((Calc_s*RCHTARGETOFFSET)+Stack)%60
                ;((Calc_s*RCHTARGETOFFSET)+Stack)/60 =>Stack
                ld  L,[UTL_Mult_s]
                ld  A,[UTL_Mult]
                mlt
                pop IX
                add HL,IX
                ld  a,#60
                div
                ld  [UTL_Mult_s],H
                ld  H,#0
                push HL

                ;Calc_s=((Calc_m*RCHTARGETOFFSET)+Stack)%60
                ;((Calc_m*RCHTARGETOFFSET)+Stack)/60 =>Stack
                ld  L,[UTL_Mult_m]
                ld  A,[UTL_Mult]
                mlt
                pop IX
                add HL,IX
                ld  a,#60
                div
                ld  [UTL_Mult_m],H
                ld  H,#0
                push HL

                ;Calc_h=((Calc_h*RCHTARGETOFFSET)+Stack)%60
                ld  L,[UTL_Mult_h]
                ld  A,[UTL_Mult]
                mlt
                pop IX
                add HL,IX
                ld  [UTL_Mult_h],L
                
                pop all
                push all
                
                ld  IX,IY
                ld  B, #4
                ld  IY,#UTL_Mult_Hu
                UTL_COPY_IYREG_TO_IXREG
                
                pop all
                
                ret

                
                
Split_Hu:       db 1
Split_S:        db 1
Split_m:        db 1
Split_h:        db 1

