;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchdb.asm
; Purpose      : RChrono Application Database State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchdb'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchDbStateManager
; Description : RChrono Application Database State Manager.
; Assumptions : 
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchDbStateManager

rchDbStateManager:

                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchDbStateStateEntryEvent

                ; Check if mode depress event.
                cp      A, #RCH_MODEDEPRESS
                jr      Z, rchDbStateModeDepressEvent

                ; Check if crown set event.
                cp      A, #RCH_CROWNSET
                jr      Z, rchDbPulledCrownEvent

                ; Check if start/split depress event.
                cp      A, #RCH_STARTSPLITDEPRESS
                jr      Z, rchDbStateStartSplitDepressEvent

                ; Check if CW pulses event.
                cp      A, #RCH_CWPULSES
                jr      Z, rchDbStateCWPulsesEvent

                ; Check if CW pulses event.
                cp      A, #RCH_CCWPULSES
                jr      Z, rchDbStateCCWPulsesEvent
                
                ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;------------------------------------------------------------------------------
rchDbStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************

                ; Enable pulse mode to change distance.
                CORE_ENABLE_PULSE_MODE
                
                ; allow switch releases to be passed to this current state
                CORE_ENABLE_SWITCH_RELEASE

                ; Set Record counter to 0
                ld      A,#0
                ld      [RecCounter],A
                
rchDbSubStateEntry:
                ;-- Test if we want to write or read a record --
                ld      A,[RCHTmpDbWrite]
                cp      A,#0
                jr      Z,rchReadDb
                cp      A,#2    
                jr      NZ,rchWriteDb
                
                ;-- Called if the stopwatch was stopped --
                ;copy stopwatch data to buffer
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                add     IY,#RCHSPLIT_HU
                KSTP_COPY_RESOURCE_TO_BUFFER
                
                ;Copy Target distance in Distance
                ld      IY, [CORECurrentASDAddress]
                ld      A,[IY+RCHTARGETOFFSET]
                ld      [IY + RCHDISTOFFSET],A
                ;Now we continue to store data...
                
rchWriteDb:      ;-- Write the record --
                car rchOpenDatabase
                
                ld      IY, [CORECurrentASDAddress]
                
                ld      A,[IY+RCHNBREC] ;Set the record number
                add     IY, #RCHSPLIT_S ;Calculate the address of the buffer
                car rchWriteRecord      ;Write the record
                
                ;-- Increment the number of record --
                car rchDbIncNumRec

                ;-- Write the number of record --
                car rchWriteNbRecHeader

                car rchCloseDatabase
                
                ;-- return to the default state --
rchEndState:    ld      A,[RCHTmpDbWrite]
                cp      A,#1
                jr      Z,rchGotoResumeState
                    ld      A,#00
                    ld      [RCHTmpDbWrite],A
                    ld      B, #RCHDEFAULTSTATE
                    CORE_REQ_STATE_CHANGE
                    ret
rchGotoResumeState:
                ld      B, #RCHRESUMESTATE
                CORE_REQ_STATE_CHANGE
                ret


rchReadDb:      ;-- Read session... Init the display --             
                ld      HL,[RCHTmpStatus]     
                or      [HL],#bRCHClearTop
                
                ld      A,[IY+RCHNBREC]
                cp      A,#0
                jr      Z,rchEndState
                    LCD_CLR_DISPLAY
                    car rchDbRead       ;Read the record pointed by RecCounter
                    car rchDbDisplayRec ;Display the record
                ret

;------------------------------------------------------------------------------                
rchDbPulledCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN SET
                ;
                ;**************************************************************
                ld      B, #RCHSETBANNERSTATE
                CORE_REQ_STATE_CHANGE
                ret


;------------------------------------------------------------------------------
rchDbStateModeDepressEvent:
                ;**************************************************************
                ;
                ;                       MODE DEPRESS
                ;
                ;**************************************************************
                ;This is default to go to the next mode
                CORE_REQ_MODE_CHANGE_NEXT
                ret


;------------------------------------------------------------------------------
rchDbStateStartSplitDepressEvent:
                ;**************************************************************
                ;
                ;                      START/SPLIT DEPRESS
                ;
                ;**************************************************************

                ;-- play sound --
                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT
                
                ;-- Return to the default state --
                ld      B, #RCHDEFAULTSTATE
                CORE_REQ_STATE_CHANGE
                ret


;------------------------------------------------------------------------------
rchDbStateCWPulsesEvent:
                ;**************************************************************
                ;
                ;                      CW PULSES
                ;
                ;**************************************************************
                ;-- Increment RecCounter, Test RecCounter --
                ld      B,[IY+RCHNBREC]
                
                ld      A,[RecCounter]
                inc     A
                cp      A,B
                jr      NZ,CW_1
                dec     A
CW_1:           ld      [RecCounter],A

                ;-- Read the record --
                car rchDbRead
                
                ;-- Display the record --
                car rchDbDisplayRec
                ret


;------------------------------------------------------------------------------             
rchDbStateCCWPulsesEvent:
                ;**************************************************************
                ;
                ;                      CCW PULSES
                ;
                ;**************************************************************
                ;-- Decrement RecCounter, Test RecCounter --
                ld      A,[RecCounter]
                dec     A
                cp      A,#255
                jr      NZ,CCW_1
                ld      A,#0
CCW_1:          ld      [RecCounter],A

                ;-- Read the record --
                car rchDbRead
                
                ;-- Display the record --
                car rchDbDisplayRec
                ret


rchMsgDist:
                dw        LCDMAINDMLINE2COL1
                db        5, DM5_D, DM5_I, DM5_S, DM5_T, DM5_COLON

                
;------------------------------------------------------------------------------             
rchDbDisplayRec:
                ;**************************************************************
                ;
                ;                      TARGET SET DISPLAY
                ;
                ;**************************************************************
                ;-- Display split time in the line 1 --
                ld      IY,#rchDBHund
                car rchDispChrono
                
                ;-- Display distance --
                ld      IY, #rchMsgDist
                LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG

                ld      L, [rchDBDistance]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD

                ld IX, #LCDMAINDMLINE2COL29
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ;-- Display split number --
                ld      L, [RecCounter]
                inc     L
                UTL_CONVERT_HEX_TO_2DIGIT_BCD

                ;Display RCHDISTOFFSET on upper display
                ld      IX, #LCDUPPERDMCOL1
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ;-- Display the LAP icon --
                LCD_UPD_LAP_FLAG ON
                ret


;----------------------------------------------------------------------------------
; read a record into the buffer
; uses IY - record index
;----------------------------------------------------------------------------------
rchReadRecord:
                ; find the record and load it   
                ; set the buffer (passed in IY)
                ld      HL, #rchDatabaseBuffer
                ld      [DBInternalMemoryAddress], HL
                
                ; set the record number
                ld      HL, IY
                ld      [DBRecordNumber], HL
                
                ; set the record size
                ld      A, #4
                ld      [DBLengthLo], A
                
                ; read
                DB_READ_RECORD_RANDOMFIX                
                ret


;----------------------------------------------------------------------------------
; writes a record from the buffer
; imput :
;   A - record index to write
;   IY - Address of the buffer    
;----------------------------------------------------------------------------------
rchWriteRecord:
                ; find the record and load it   
                ; set the buffer (passed in IY)
                ld      [DBInternalMemoryAddress], IY
                
                ; set the record number
                ld      B,#0
                ld      [DBRecordNumber], BA
                
                ; set the record size
                ld      A, #4
                ld      [DBLengthLo], A
                
                ; write
                DB_WRITE_RECORD_RANDOMFIX               
                
                ; store that the database has been modified so it will be uploaded by the PC
                ld      A, [CORECurrentMode]
                CORE_DATABASE_MODIFIED_BY_USER
                ret

                
;------------------------------------------------------------
; Read a record in the buffer
;------------------------------------------------------------
rchDbRead:
                car rchOpenDatabase
                
                ld      B,#0
                ld      A,[RecCounter]
                ld      IY,BA
                car rchReadRecord
                
                car rchCloseDatabase
                ret

                
;------------------------------------------------------------
rchDbIncNumRec:
                ;**************************************************************
                ;
                ;  Increment the number of record   
                ;
                ;  Note: I test only the low byte because the maximum number of record is 99
                ;**************************************************************
                ld      IY, [CORECurrentASDAddress]
                
                ld      A,[IY+RCHNBREC]
                cp      A,#99 ;[rchHTNumRecLo]
                jr      Z,rchIncEnd

                inc     A
                ld      [IY+RCHNBREC],A
rchIncEnd:      ld      [rchHNumRec],A
                ret
                
                

;**************************************************************
;   Display Chrono function
;   Input :
;       . IY = Address of the bloc to display
;           IY+0 : Hundredths (not displayed but necessary)
;           IY+1 : Second in BCD
;           IY+2 : Minute in BCD
;           IY+3 : Hour in BCD
;
;**************************************************************
rchDispChrono:
                ; Display the Character :
                ld L, #DM5_COLON
                ld IX, #LCDMAINDMLINE1COL11
                LCD_DISP_SMALL_FIXED_WIDTH_DM_CHAR

                ; Display the Character .
                ld L, #DM5_PERIOD
                ld IX, #LCDMAINDMLINE1COL25
                LCD_DISP_SMALL_FIXED_WIDTH_DM_CHAR

                ; this will be displayed hour
                ld A, [IY + 3]
                ld IX, #LCDMAINDMLINE1COL1
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ; this will be displayed minute
                ld A, [IY + 2]
                ld IX, #LCDMAINDMLINE1COL15
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ; this will be displayed second
                ld A, [IY + 1]
                ld IX, #LCDMAINDMLINE1COL29
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP

                ret

                
                
;------------------------------------------------------------
; Internal variables

RecCounter:     db  00
