;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchdef.asm
; Purpose      : RChrono Application Default State Manager
;
; Version      : 1.7
; Autor        : Pierre DELORE
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchdef'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchDefaultStateManager
; Description : RChrono Application Default State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchDefaultStateManager

rchDefaultStateManager:

                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]
                
                ; Set RCHTmpStatus with CORECurrentASDAddress+RCHSTATUS
                ld      HL,IY
                add     HL,#RCHSTATUS
                ld      [RCHTmpStatus],HL
                
                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchDefaultStateStateEntryEvent

                ; Check if mode depress event.
                cp      A, #RCH_MODEDEPRESS
                jr      Z, rchDefaultStateModeDepressEvent

                ; Check if crown set event.
                cp      A, #RCH_CROWNSET
                jr      Z, rchDefaultPulledCrownEvent

                ; Check if start/split depress event.
                cp      A, #RCH_STARTSPLITDEPRESS
                jr      Z, rchDefaultStateStartSplitDepressEvent

                ; Check if stop/reset depress event.
                cp      A, #RCH_STOPRESETDEPRESS
                jr      Z, rchDefaultStateStopResetDepressEvent
                
                ; Check if stop/reset release event.
                cp      A, #RCH_STOPRESETRELEASE
                jr      Z, rchDefaultStateStopResetReleaseEvent
                
                ; Check if CW pulses event.
                cp      A, #RCH_CWPULSES
                jr      Z, rchDefaultStateCWPulsesEvent

                ; Check if CW pulses event.
                cp      A, #RCH_CCWPULSES
                jr      Z, rchDefaultStateCCWPulsesEvent
                
                ; Check if timeout hi-res done event.
                cp      A, #RCH_TIMEOUTDONEHIGHRES
                jr      Z, rchDefaultStateTimeoutHiResDoneEvent
                
                ; Check if Stopwatch display update event
                cp      A,#RCH_DISPLAY_UPDATE_STPRES
                jr      Z,rchDisplayUpdateStpRes

                ; Check if TOD display update event
                cp      A,#RCH_DISPLAY_UPDATE_TODRES
                jr      Z,rchDisplayUpdateTODRes
                
                ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;------------------------------------------------------------------------------
rchDefaultStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************

                ; Enable pulse mode to change distance.
                CORE_ENABLE_PULSE_MODE

                ; allow switch releases to be passed to this current state
                CORE_ENABLE_SWITCH_RELEASE
                
                ; Hide the chrono icon
                LCD_UPD_CHR_FLAG OFF
                
                ;request that any updates done to the stopwatch at 16HZ interval that
                ;a event be passed to the foreground application for display updates
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_ENABLE_DISP_UPD_EVENT

                ld  a,#00
                ld  [RCHTmpAlarm],A
                
                ;Set the return state if there is a popup 
                ;RCHDTOPSTATE because it's necessary to display the top line
                CORE_SET_POPDOWN_STATE RCHDTOPSTATE
                
                ;enable seconds updates from the primary time zone resource
                ld      A, [COREPTZIndex]
                KTOD_ENABLE_DISP_UPD_MIN_EVENT
                
                ld      A,#0    
                ld      [RCHTmpDbWrite],A
                
            ;-- Read the header --
                car rchReadHeader
                
rchDefaultStateStopResetReleaseEvent:
rchDefaultSubStateEntry:
            ;-- Display the chrono value --
                car rchDisplayChrono
                
                
                ;Test if it's necessary to clear the display
                ;ld      IY, [CORECurrentASDAddress]
                ld      HL,[RCHTmpStatus]
                bit     [HL],#bRCHClearTop
                jr      Z,rchLabel1
                    LCD_CLR_DISPLAY
                    jr  rchLabel2
                    
rchLabel1:      ;Test if it's necessary to call the top display state
                bit     [HL],#bRCHRefreshTop
                jr      Z,rchExitDefSubStateEntry
rchLabel2:          ld      B, #RCHDTOPSTATE
                    CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY
                
rchExitDefSubStateEntry:
                ret

                
;------------------------------------------------------------------------------                
rchDefaultPulledCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN SET
                ;
                ;**************************************************************                
                ;Call the state
                ld      B, #RCHSETBANNERSTATE
                CORE_REQ_STATE_CHANGE
                ret

                
;------------------------------------------------------------------------------
rchDefaultStateModeDepressEvent:
                ;**************************************************************
                ;
                ;                       MODE DEPRESS
                ;
                ;**************************************************************

                ;This is default to go to the next mode
                CORE_REQ_MODE_CHANGE_NEXT
                ret
                
                
;------------------------------------------------------------------------------
rchDefaultStateStartSplitDepressEvent:
                ;**************************************************************
                ;
                ;                      START/SPLIT DEPRESS
                ;
                ;**************************************************************
                ; Check the chrono is running
                ; get resource status
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_GET_RESOURCE_STATUS
                cp      A, #22H
                jr      Z, StopwatchNotResetAndRunning
StopwatchResourceIsReset:
StopwatchNotResetAndStopped:                
                
                ;start stopped stopwatch resource
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_START_RESOURCE

                ;Write Time/Date/Target Distance in the database header
                car     rchWriteInitHeader
                
                ;request that any updates done to the stopwatch at 16HZ interval that
                ;a event be passed to the foreground application for display updates
                KSTP_ENABLE_DISP_UPD_EVENT

                ;Set ON the Chrono flag in the TOD
                ld A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE CHR_RSRC_FLAG,FLAG_ON

                ;play sound
                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT
                
                ld      HL,[RCHTmpStatus]
                bit     [HL],#bRCHAutoDist
                jr      Z,ssL1
                    ld      A,#00
                    ld      IY, [CORECurrentASDAddress]
                    ld      [IY + RCHDISTOFFSET],A
ssL1:                
                ret

StopwatchNotResetAndRunning:
                ; Take split, Display Split, Calculate the target and display the target time

                ld      B, #4
                add     IY,#RCHSPLIT_HU
                ld      IX,#RCHTmp_HU
                UTL_COPY_IYREG_TO_IXREG
                
                ld      IY, [CORECurrentASDAddress]

                ;copy stopwatch data to buffer
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                add     IY,#RCHSPLIT_HU
                KSTP_COPY_RESOURCE_TO_BUFFER
                
                ;play sound
                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ld      B, #RCHTARGETSTATE
                CORE_REQ_STATE_CHANGE
                ret
                
                
;------------------------------------------------------------------------------                
rchDefaultStateStopResetDepressEvent:
                ;**************************************************************
                ;
                ;                      STOP/RESET DEPRESS
                ;
                ;**************************************************************

                ;Set the flag to clear the display and display the top display
                ld      HL,[RCHTmpStatus]
                or      [HL],#bRCHClearTop
                
                ; Check if the chrono is running

                ;get resource status
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_GET_RESOURCE_STATUS
                cp      A, #22H
                jr      Z, rchDefaultStopResetDepressStopChrono
                
                ;-- The chrono is stopped. Display "HOLD TO RESET">2s> Reset chrono --               
                
                ; Request 2sec timeout.
                CORE_REQ_TIMEOUT_HIRES  RCHHIRESTO_2SECONDS

                ;Clear the display and display "HOLD TO RESET"
                LCD_CLR_DISPLAY
                LCD_DISP_SMALL_DM_MSG_HOLD_TO_RESET
                ret
                
rchDefaultStopResetDepressStopChrono:
                
                ;-- Stop the chrono and store time --
                ld      A,#0    
                ld      [IY + RCHDISPLAYTOP],A

                ;stop a running stopwatch resource
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_STOP_RESOURCE

                ;Set OFF the Chrono flag in the TOD
                ld A, [CORECurrentMode]
                LCD_UPDATE_TOD_FLAG_RESOURCE_STATE CHR_RSRC_FLAG,FLAG_OFF

                ;play sound
                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ;Display the chrono value
                car     rchDisplayChrono

                ;-- Goto to the database state to store the stop time --
                ld      A,#2    
                ld      [RCHTmpDbWrite],A
                
                ld      B, #RCHDBSTATE
                CORE_REQ_STATE_CHANGE               
                ret


;------------------------------------------------------------------------------
rchDefaultStateCWPulsesEvent:
                ;**************************************************************
                ;
                ;                      CW PULSES
                ;
                ;**************************************************************
                ;-- Stop Drink or Eat alarm --
                AUDSTOP_MELODY
                
                ;Test if the chrono is running
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_GET_RESOURCE_STATUS
                cp      A, #22H
                jr      Z, rchDisplayCW   ; StopwatchNotResetAndRunning;
                    ld      B, #RCHDBSTATE
                    CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY
                    ret
                
rchDisplayCW:   ; Next Display
                ld      HL,[RCHTmpStatus]
                bit     [HL],#bRCHEatMsg
                jr      NZ,rchDisplay2
                    
                bit     [HL],#bRCHDrinkMsg
                jr      NZ,rchDisplay2
                    
rchDCW2:        ld      A,[IY+RCHDISPLAYTOP]
                inc     A
                cp      A,#04
                jr      NZ,rchDisplayCWexit
                    ld      A,#00
rchDisplayCWexit:
                ld      [IY+RCHDISPLAYTOP],A
                jr      rchDisplay2 

                
;------------------------------------------------------------------------------             
rchDefaultStateCCWPulsesEvent:
                ;**************************************************************
                ;
                ;                      CCW PULSES
                ;
                ;**************************************************************
                AUDSTOP_MELODY
                ;Test if the chrono is running
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_GET_RESOURCE_STATUS
                cp      A, #22H
                jr      Z, rchDisplayCCW   ; StopwatchNotResetAndRunning
                    ld      B, #RCHDBSTATE
                    CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY
                    ret
                
                
rchDisplayCCW:  ; Next Display
                ld      HL,[RCHTmpStatus]
                bit     [HL],#bRCHEatMsg
                jr      NZ,rchDisplay2
                
                bit     [HL],#bRCHDrinkMsg
                jr      NZ,rchDisplay2

rchDCCW2:       
;                ld      A,[IY+RCHDISPLAYTOP]
;                dec     A
;                cp      A,#255
;                jr      nz,rchDisplayCCWexit
;                    ld      A,#03
                ld      A,[IY+RCHPREFDISP]
rchDisplayCCWexit:
                ld      [IY+RCHDISPLAYTOP],A
                
rchDisplay2:    ld      B, #RCHDTOPSTATE
                CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY    
                ret
                
                
;------------------------------------------------------------------------------             
rchDefaultStateTimeoutHiResDoneEvent:
                ;**************************************************************
                ;
                ;                      TIMEOUT DONE HI-RES
                ;
                ;**************************************************************

                ; Cancel current switch release.  Not needed in this state.
                HW_KBD_CANCEL_CURRENT_SWITCH_RELEASE
                                
                AUDSTART_SYSTEM_MELODY AUDSWBEEPMELODY, AUDNOMELODYDONEEVENT

                ;Clear a set of value
                ld      B, #(RCHSTATUS-RCHSPLIT_HU)
                ld      HL,IY
                add     HL,#RCHSPLIT_HU
                KRES_CLEAR_N_BYTES
                
                ld      A,#1
                ld      [IY + RCHDISTOFFSET],A

                ;reset the stopwatch
                ld      A, [IY + RCHSTOPWATCHRESOURCE0OFFSET]
                KSTP_RESET_RESOURCE
                
                ld      HL,[RCHTmpStatus]
                or      [HL],#bRCHClearTop
                
                ; Redisplay everything.
                jr      rchDefaultSubStateEntry


;------------------------------------------------------------------------------                
rchDisplayUpdateStpRes:
                ;**************************************************************
                ;
                ;                      Display update stopwatch event
                ;
                ;**************************************************************
                car     rchDisplayChrono
                
                ;Show the chrono icon
                LCD_UPD_CHR_FLAG ON
                
                ;Second=0 ?
                ld  A,[IY+1]
                cp  A,#00
                jr  Z,LUStp10
                    ld  a,#00
                    ld  [RCHTmpAlarm],A
                    jr  LUStpEnd
                
LUStp10:        ld  A,[RCHTmpAlarm]
                cp  A,#00
                jr  NZ,LUStpEnd
                    ;Convert Time from BCD to HEX 
                    ld  IX,IY
                    UTL_CONVERT_4BYTE_FROM_BCD_TO_HEX
                    ;IY+0 : Hundredths (not displayed but necessary)
                    ;IY+1 : Second in BCD
                    ;IY+2 : Minute in BCD
                    ;IY+3 : Hour in BCD
                    
                    ;-- HL=Hour*60
                    ld  A,[IY+3]
                    ld  L,#60
                    mlt ;HL=L*A
                    
                    ;-- HL=HL+Minute
                    ld      A,[IY+2]
                    ld      B,#00
                    add     HL,BA
                    ld      IX,HL
                    
                    ;-- Test if (Hour*60+Minute)=0 -Yes-> End
                    cp      HL,#0
                    jr      Z,LUStpEnd
                    
                    ld  IY,[CORECurrentASDAddress]

                    ld      HL,[RCHTmpStatus]
                    bit     [HL],#bRCHDrink
                    jr      Z,LUStp2
                        ld  HL,IX
                        ld  A,[IY+RCHDRINKV]
                        div ;L=HL/A  H=Reste
                        cp  H,#00
                        jr  nz,LUStp2
                            ld  HL,[RCHTmpStatus]
                            or  [HL],#bRCHDrinkMsg
                            ld  B,#04
                            
LUStp2:             ld      HL,[RCHTmpStatus]
                    bit     [HL],#bRCHEat
                    jr      Z,LUStpAction
                        ld  HL,IX
                        ld  A,[IY+RCHEATV]
                        div ;L=HL/A  H=Reste
                        cp  H,#00
                        jr  nz,LUStpAction ;LUStpEnd
                            ld  HL,[RCHTmpStatus]
                            or  [HL],#bRCHEatMsg
                            ld  B,#05
                            
LUStpAction:        cp  B,#00
                    jr  Z,LUStpEnd
                        ld  [RCHTmpAlarm],B
                        ;ld  [IY+RCHDISPLAYTOP],B
                        
                        AUDSTART_SYSTEM_MELODY AUDTIMERMELODY, AUDNOMELODYDONEEVENT
                        
                        ld      B, #RCHDTOPSTATE
                        CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY    
LUStpEnd:       
                ret


;------------------------------------------------------------------------------             
rchDisplayUpdateTODRes:
                ;**************************************************************
                ;
                ;                      Display update TOD event
                ;
                ;**************************************************************
                ld      B, #RCHDTOPSTATE
                CORE_REQ_STATE_CHANGE_NO_CLEAR_DISPLAY
                ret
