;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchset.asm
; Purpose      : RChrono Application Set State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchset'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchSetStateManager
; Description : RChrono Application Set State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchSetStateManager

rchSetStateManager:

                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchSetStateEntryEvent

                ; Check if stop/reset depress event.
                cp      A, #RCH_STOPRESETDEPRESS
                jr      Z, rchSetStateStopResetDepressEvent
                
                cp      A, #RCH_STARTSPLITDEPRESS
                jr      Z, rchSetStateStartSplitDepressEvent                
                
                cp      A, #RCH_CWPULSES
                jr      Z, rchSetStateCWPulseEvent

                cp      A, #RCH_CCWPULSES
                jr      Z, rchSetStateCCWPulseEvent

                cp      A, #RCH_CROWNHOME
                jr      Z, rchSetStatePushedCrownEvent

                ret
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;------------------------------------------------------------
rchSetStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************
                ; Initialize
                ld      A,#00
                ld      [rchSetSel],A
                
                ; Set the index to display the target setting 
                ld      A,#00
                ld      [RCHTmpSetIndex],A
                
                car         rchwaSetTargetDisplay
                car         rchDisplaySetTarget

                ; Set the blinking service
                LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR rchDisplaySetTarget
                LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR rchDisplayClrTarget
                CORE_REQ_BLINK_4HZ
                
                ; Enable pulse mode to change values.
                CORE_ENABLE_PULSE_MODE
                ret

                
;------------------------------------------------------------
rchSetStateStartSplitDepressEvent:
                ;**************************************************************
                ;
                ;                       START/SPLIT
                ;
                ;**************************************************************
                ld      HL,[RCHTmpStatus]
                
                ld      A,[RCHTmpSetIndex]
                cp      A,#01
                jr      Z,LssDrink
                cp      A,#02
                jr      Z,LssEat
                
                ;-- Toggle Target distance 1 to 1/10 --
                ld      HL,#rchSetSel
                xor     [HL],#01
                ret
                
                ;-- Toggle Drink timer on/off --
LssDrink:       xor     [HL],#bRCHDrink
                LCD_CLR_DISPLAY
                car     rchDrinkDisplay
                car     rchSetDrinkDisplay
                ret

                ;-- Toggle Eat timer on/off --
LssEat:         xor     [HL],#bRCHEat
                LCD_CLR_DISPLAY
                car     rchEatDisplay
                car     rchSetEatDisplay
                ret

                
;------------------------------------------------------------
rchSetStateStopResetDepressEvent:
                ;**************************************************************
                ;
                ;                       STOP/RESET
                ;
                ;**************************************************************
                LCD_CLR_DISPLAY
                ;-- Select next setting --
                ld      A,[RCHTmpSetIndex]
                inc     A
                cp      A,#04
                jr      nz,Lsr1
                    ld      A,#00
                    
Lsr1:           ld      [RCHTmpSetIndex],A
                cp      A,#00
                jr      nz,Lsr2
                
                    ;-- Target distance setting --
                    car         rchwaSetTargetDisplay
                    car         rchDisplaySetTarget
                    ; Set the blinking service
                    LCD_WRITE_4HZ_GEN_BLINK_DISP_ROUTINE_ADDR rchDisplaySetTarget
                    LCD_WRITE_4HZ_GEN_BLINK_CLR_ROUTINE_ADDR rchDisplayClrTarget
                    CORE_REQ_BLINK_4HZ
                    jr      LsrEnd
                    
Lsr2:           cp      A,#01
                jr      nz,Lsr3
                
                    ;-- Drink timer setting --
                    car     rchDrinkDisplay
                    car     rchSetDrinkDisplay
                    CORE_CANCEL_BLINK_4HZ
                    jr      LsrEnd
                    
Lsr3:           cp      A,#02
                jr      nz,Lsr4
                
                    ;-- Eat timer setting --
                    car     rchEatDisplay
                    car     rchSetEatDisplay
                    CORE_CANCEL_BLINK_4HZ
                    jr      LsrEnd
Lsr4:
                cp      A,#03
                jr      nz,LsrEnd
                
                    ;-- Go to the next SET state --
                    ld      B, #RCHSETSTATE2
                    CORE_REQ_STATE_CHANGE                    
LsrEnd:
                ret

                
;------------------------------------------------------------
rchSetStatePushedCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN HOME
                ;
                ;**************************************************************
                ld      HL,[RCHTmpStatus]
                or      [HL],#02
                
                ld      B, #RCHDEFAULTSTATE
                CORE_REQ_STATE_CHANGE
                ret

                
;------------------------------------------------------------
rchSetStateCWPulseEvent:
                ;**************************************************************
                ;
                ;                       CLOCKWISE CROWN ROTATION
                ;
                ;**************************************************************
                ld      HL,IY
                
                ld      A,[RCHTmpSetIndex]
                cp      A,#01
                jr      Z,Lcwsd
                cp      A,#02
                jr      Z,Lcwse
   
                ld      A,[rchSetSel]
                cp      A,#00
                jr      NZ,Lcw10
                    ;-- Increase target distance --
                    ld      A,[IY + RCHTARGETOFFSET]
                    inc     A
                    cp      A,#100
                    jr      nz,Lcw100E
                        ld      a,#01
Lcw100E:            ld      [IY + RCHTARGETOFFSET],A
                    jr      LcwEnd
                
                ;-- Increase 1/10 target distance --
Lcw10:          ld      A,[IY + RCHTARGET10]
                inc     A
                cp      A,#10
                jr      nz,Lcw10E
                    ld      a,#00
Lcw10E:         ld      [IY + RCHTARGET10],A
           
LcwEnd:         car rchDisplaySetTarget
                ret
                
                ;-- Increase Drink timer --
Lcwsd:          ld      A,[IY + RCHDRINKV]
                inc     A
                cp      A,#100
                jr      nz,Lcwsd1
                    ld  A,#01
Lcwsd1:         ld      [IY + RCHDRINKV],A
                car     rchSetDrinkDisplay
                ret

                ;-- Increase Eat Timer --
Lcwse:          add     HL,#RCHEATV
                inc     [HL]
                cp      [HL],#100
                jr      nz,Lcwse1
                    ld  [HL],#01
Lcwse1:         car     rchSetEatDisplay
                ret
                
;------------------------------------------------------------
rchSetStateCCWPulseEvent:
                ;**************************************************************
                ;
                ;                       COUNTER CLOCKWISE CROWN ROTATION
                ;
                ;**************************************************************
                ld      HL,IY

                ld      A,[RCHTmpSetIndex]
                cp      A,#01
                jr      Z,Lcwwsd
                cp      A,#02
                jr      Z,Lcwwse
                
            ;-- Decrease Target distance --
                ld      A,[rchSetSel]
                cp      A,#00
                jr      NZ,Lccw10
                    ld      A,[IY + RCHTARGETOFFSET]
                    dec     A
                    jr      nz,Lccw100E
                        ld      a,#99
Lccw100E:           ld      [IY + RCHTARGETOFFSET],A
                jr      LccwEnd
                
            ;-- Decrease 1/10 Target distance --
Lccw10:         ld      A,[IY + RCHTARGET10]
                dec     A
                cp      A,#255
                jr      nz,Lccw10E
                    ld      a,#9
Lccw10E:        ld      [IY + RCHTARGET10],A
         
LccwEnd:        car rchDisplaySetTarget
                ret
                
            ;-- Decrease Drink value --
Lcwwsd:         add     HL,#RCHDRINKV
                dec     [HL]
                cp      [HL],#00
                jr      nz,Lcwwsd1
                    ld  [HL],#99
Lcwwsd1:        car     rchSetDrinkDisplay
                ret

            ;-- Decrease Eat value --
Lcwwse:         add     HL,#RCHEATV
                dec     [HL]
                cp      [HL],#00
                jr      nz,Lcwwse1
                    ld  [HL],#99
Lcwwse1:        car     rchSetEatDisplay
                ret
                

;------------------------------------------------------------
; Display the target distance 'xx.x'
;
rchDisplaySetTarget:
                ;Read RCHDISTOFFSET and convert to BCD
                ld      IY, [CORECurrentASDAddress]
                ld      L, [IY + RCHTARGETOFFSET]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD

                ;Display RCHDISTOFFSET on upper display
                ld      IX, #LCDMAINDMLINE2COL24
                LCD_DISP_SMALL_FIXED_WIDTH_2DIG_DM_DATA_NO_ZERO_SUP
                
                ;Display '.'
                ld L, #DM5_PERIOD
                ld IX, #LCDMAINDMLINE2COL36
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

                ;Display RCHTARGET10
                ld      IY, [CORECurrentASDAddress]
                ld      L, [IY + RCHTARGET10]
                ld      IX, #LCDMAINDMLINE2COL38
                LCD_DISP_SMALL_FIXED_WIDTH_DM_CHAR
                
                ret

;------------------------------------------------------------
; Clear the target distance (used by the blinking function)
;
rchDisplayClrTarget:
                ld      A,[rchSetSel]
                cp      A,#00
                jr      NZ,Lclr1
                
                ld IX, #LCDMAINDMLINE2COL24
                LCD_CLR_SMALL_FIXED_WIDTH_2DIG_DM_DATA
                ret
Lclr1:                
                ;Display RCHTARGET10
                ld      L,#DM5_SPACE
                ld      IX, #LCDMAINDMLINE2COL38
                LCD_DISP_SMALL_FIXED_WIDTH_DM_CHAR
                ret


;------------------------------------------------------------
; Display the target msg
;
rchwaSetTargetDisplay:
                ld        IY,#rchwaSetDisplayLine1
                LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG

                ld        IY,#rchwaSetDisplayLine2
                LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG
                ret

rchwaSetDisplayLine1:
                dw        LCDMAINDMLINE1COL1
                db        6, DM5_T, DM5_A, DM5_R, DM5_G, DM5_E,DM5_T

rchwaSetDisplayLine2:
                dw        LCDMAINDMLINE2COL1
                db        5, DM5_D, DM5_I, DM5_S, DM5_T,DM5_EQUAL


rchSetSel:      db  1



;------------------------------------------------------------
; Display EAT or DRINK timer value in the segment area
;
rchSetEatDisplay:
                ld  HL,#RCHEATV
                jr  SetDisplay
rchSetDrinkDisplay:
                ld HL,#RCHDRINKV
SetDisplay:
                ld  IY,[CORECurrentASDAddress]
                add IY,HL
                ld  L,[IY]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD
                ld IX, #LCDSEGDIGIT1
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                ret

                
;------------------------------------------------------------
; Display the EAT msg
;
rchEatDisplay:
                ld  IY,#msgSetEat
                LCD_DISP_BANNER_MSG
                 
                ld  B,#bRCHEat
                
                jr  DispBann
                
;------------------------------------------------------------
; Display the DRINK msg
;
rchDrinkDisplay:
                ld  IY,#msgSetDrink
                LCD_DISP_BANNER_MSG

                ld  B,#bRCHDrink

DispBann:       ld  HL, [RCHTmpStatus]
                ld  A,[HL]
                bit A,B
                jr  NZ,LsdOn
                    ld  IY,#msgSetOff
                    jr  LsdDisp
LsdOn:
                ld  IY,#msgSetOn
LsdDisp:        LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG
                ld IY, #msgSetMin
                LCD_DISP_SEG_LINE_MSG
                ret

;------------------------------------------------------------
msgSetEat:
                db LCDBANNER_COL1
                db DM5_E, DM5_A, DM5_T
                db LCDBANNER_COL1
                db DM5_T, DM5_I, DM5_M, DM5_E, DM5_R
                db LCD_END_BANNER


;------------------------------------------------------------
msgSetDrink:
                db LCDBANNER_COL1
                db DM5_D, DM5_R, DM5_I, DM5_N, DM5_K
                db LCDBANNER_COL1
                db DM5_T, DM5_I, DM5_M, DM5_E, DM5_R
                db LCD_END_BANNER
                
;------------------------------------------------------------
msgSetMin:      db SEG_SPACE, SEG_SPACE, SEG_SPACE, SEG_M, SEG_I, SEG_N

;------------------------------------------------------------
msgSetOn:       dw LCDMAINDMLINE2COL28
                db 3, DM5_O, DM5_N, DM5_SPACE

;------------------------------------------------------------
msgSetOff:      dw LCDMAINDMLINE2COL28
                db 3, DM5_O, DM5_F, DM5_F

