;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchset2.asm
; Purpose      : RChrono Application Set State Manager (second file)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchset2'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchSetStateManager
; Description : RChrono Application Set State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchSetState2Manager

rchSetState2Manager:

                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchSetState2EntryEvent

                ; Check if stop/reset depress event.
                cp      A, #RCH_STOPRESETDEPRESS
                jr      Z, rchSetState2StopResetDepressEvent
                
                cp      A, #RCH_STARTSPLITDEPRESS
                jr      Z, rchSetState2StartSplitDepressEvent                
                
                cp      A, #RCH_CWPULSES
                jr      Z, rchSetState2CWPulseEvent

                cp      A, #RCH_CCWPULSES
                jr      Z, rchSetState2CCWPulseEvent

                cp      A, #RCH_CROWNHOME
                jr      Z, rchSetState2PushedCrownEvent

                ret
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;------------------------------------------------------------
rchSetState2EntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************
                
                car     rchAutoDistDisplay
                car     rchSetAutoDistDisplay

                CORE_CANCEL_BLINK_4HZ
                
                ; Enable pulse mode to change values.
                CORE_ENABLE_PULSE_MODE
                ret


;------------------------------------------------------------
rchSetState2StartSplitDepressEvent:
                ;**************************************************************
                ;
                ;                       START/SPLIT
                ;
                ;**************************************************************
                ld      HL,[RCHTmpStatus]
                
                ld      A,[RCHTmpSetIndex]
                cp      A,#03
                jr      Z,LssAutoDist
                ret
                
                ;-- Toggle Auto-Distance on/off --
LssAutoDist:    xor     [HL],#bRCHAutoDist
                LCD_CLR_DISPLAY
                car     rchAutoDistDisplay
                car     rchSetAutoDistDisplay
                ret


;------------------------------------------------------------
rchSetState2StopResetDepressEvent:
                ;**************************************************************
                ;
                ;                       STOP/RESET
                ;
                ;**************************************************************
                LCD_CLR_DISPLAY
                ;-- Select next setting --
                ld      A,[RCHTmpSetIndex]
                inc     A
                cp      A,#05
                jr      nz,Lsr1
                    ld      A,#00
                    
Lsr1:           ld      [RCHTmpSetIndex],A
                cp      A,#00
                jr      nz,Lsr2
                
                 ;-- Go to the first SET state --
                    ld      B, #RCHSETSTATE
                    CORE_REQ_STATE_CHANGE
                    ret
                    
Lsr2:           cp      A,#04
                jr      nz,Lsr3
                    car     rchPrefDispDisplay
                    ret
Lsr3:

LsrEnd:
                ret

                
;------------------------------------------------------------
rchSetState2PushedCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN HOME
                ;
                ;**************************************************************
                ld      HL,[RCHTmpStatus]
                or      [HL],#02
                
                ld      B, #RCHDEFAULTSTATE
                CORE_REQ_STATE_CHANGE
                ret

                
;------------------------------------------------------------
rchSetState2CWPulseEvent:
                ;**************************************************************
                ;
                ;                       CLOCKWISE CROWN ROTATION
                ;
                ;**************************************************************
                ld      HL,IY
                
                ld      A,[RCHTmpSetIndex]
                cp      A,#03
                jr      Z,Lcwsad
                cp      A,#04
                jr      Z,Lcwspd
                ret   
                
                ;-- Increase Auto-Distance value --
Lcwsad:         ld      A,[IY + RCHAUTODISTV]
                inc     A
                cp      A,#100
                jr      nz,Lcwsad1
                    ld  A,#01
Lcwsad1:        ld      [IY + RCHAUTODISTV],A
                car     rchSetAutoDistDisplay
                ret
                
                ;-- Next pref display --
Lcwspd:         ld      A,[IY + RCHPREFDISP]
                inc     A
                cp      A,#04
                jr      nz,Lcwspd1
                    ld  A,#00
Lcwspd1:        ld      [IY + RCHPREFDISP],A
                car     rchPrefDispDisplay
                ret
                
                
;------------------------------------------------------------
rchSetState2CCWPulseEvent:
                ;**************************************************************
                ;
                ;                       COUNTER CLOCKWISE CROWN ROTATION
                ;
                ;**************************************************************
                ld      HL,IY

                ld      A,[RCHTmpSetIndex]
                cp      A,#03
                jr      Z,Lcwwsad
                cp      A,#04
                jr      Z,Lcwwspd
                ret
                
                ;-- Decrease Auto-Distance value --
Lcwwsad:        add     HL,#RCHAUTODISTV
                dec     [HL]
                cp      [HL],#00
                jr      nz,Lcwwsad1
                    ld  [HL],#99
Lcwwsad1:       car     rchSetAutoDistDisplay
                ret

                ;-- Precedent pref display --
Lcwwspd:        add     HL,#RCHPREFDISP
                dec     [HL]
                cp      [HL],#255
                jr      nz,Lcwwspd1
                    ld  [HL],#03
Lcwwspd1:       car     rchPrefDispDisplay
                ret
                

;------------------------------------------------------------
; Display AUTO-DISTANCE value in the segment area
;
rchSetAutoDistDisplay:
                ld  HL,#RCHAUTODISTV
SetDisplay:
                ld  IY,[CORECurrentASDAddress]
                add IY,HL
                ld  L,[IY]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD
                ld IX, #LCDSEGDIGIT5
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                ret

                
;------------------------------------------------------------
; Display the AUTO DIST msg
;
rchAutoDistDisplay:
                ld  IY,#msgSetAutoDist
                LCD_DISP_BANNER_MSG

                ld  B,#bRCHAutoDist

DispBann:       ld  HL, [RCHTmpStatus]
                ld  A,[HL]
                bit A,B
                jr  NZ,LsdOn
                    ld  IY,#msgSetOff
                    jr  LsdDisp
LsdOn:
                ld  IY,#msgSetOn
LsdDisp:        LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG

                ld IY, #msgVal
                LCD_DISP_SEG_LINE_MSG

                LCD_UPD_COLON2_FLAG ON
                ret

                
;------------------------------------------------------------
; Display the PREF DISP msg
;
rchPrefDispDisplay:
                LCD_CLR_DISPLAY

                ld IY, #msgPrefDisp
                LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG
                
                ld L, #DM5_EQUAL
                ld IX, #LCDMAINDMLINE2COL3
                LCD_DISP_SMALL_PROP_WIDTH_DM_CHAR

                ld  IY,[CORECurrentASDAddress]
                ld  A,[IY + RCHPREFDISP]
                cp  A,#00
                jr  NZ,LPD1
                    ld  IY,#msgPrefTimeMsg
                    jr  LPDMsg
LPD1:           cp  A,#01
                jr  NZ,LPD2
                    ld  IY,#msgPrefTargetMsg
                    jr  LPDMsg
LPD2:           cp  A,#02
                jr  NZ,LPD3
                    ld  IY,#msgPrefSplitMsg
                    jr  LPDMsg
LPD3:           cp  A,#03
                jr  NZ,LPDMsg
                    ld  IY,#msgPrefLapMsg
                    jr  LPDMsg
LPDMsg:         LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG
                ret
                
;------------------------------------------------------------
msgSetAutoDist:
                db LCDBANNER_COL1
                db DM5_A, DM5_U, DM5_T, DM5_O, DM5_SPACE,DM5_D, DM5_I, DM5_S, DM5_T
                db LCDBANNER_COL20
                db DM5_EQUAL
                db LCD_END_BANNER
                
;------------------------------------------------------------
msgSetOn:       dw LCDMAINDMLINE2COL28
                db 3, DM5_O, DM5_N, DM5_SPACE

;------------------------------------------------------------
msgSetOff:      dw LCDMAINDMLINE2COL28
                db 3, DM5_O, DM5_F, DM5_F

;------------------------------------------------------------
msgVal:         db SEG_SPACE, SEG_V, SEG_A, SEG_L, SEG_SPACE, SEG_SPACE

;------------------------------------------------------------
msgPrefDisp:    dw LCDMAINDMLINE1COL1
                db 9, DM5_P, DM5_R, DM5_E, DM5_F, DM5_SPACE, DM5_D, DM5_I, DM5_S, DM5_P

;------------------------------------------------------------
msgPrefTimeMsg:
                dw LCDMAINDMLINE2COL11
                db 4, DM5_T, DM5_I, DM5_M, DM5_E

;------------------------------------------------------------
msgPrefTargetMsg:
                dw LCDMAINDMLINE2COL11
                db 6, DM5_T, DM5_A, DM5_R, DM5_G, DM5_E, DM5_T

;------------------------------------------------------------
msgPrefSplitMsg:
                dw LCDMAINDMLINE2COL11
                db 5, DM5_S, DM5_P, DM5_L, DM5_I, DM5_T

;------------------------------------------------------------
msgPrefLapMsg:
                dw LCDMAINDMLINE2COL11
                db 3, DM5_L, DM5_A, DM5_P

