;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rchTarget.asm
; Purpose      : RChrono Application Target State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rchtarget'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rchTargetStateManager
; Description : RChrono Application Target State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rchTargetStateManager

rchTargetStateManager:
                ; Set IYReg the address of the RChrono ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RCH_STATEENTRY
                jr      Z, rchTargetStateStateEntryEvent

                ; Check if start/split depress event.
                cp      A, #RCH_STARTSPLITDEPRESS
                jr      Z, rchTargetStateStartSplitDepressEvent

                ; Check if CW pulses event.
                cp      A, #RCH_CWPULSES
                jr      Z, rchTargetStateCWPulsesEvent

                ; Check if CW pulses event.
                cp      A, #RCH_CCWPULSES
                jr      Z, rchTargetStateCCWPulsesEvent
                
                ; Check if timeout hi-res done event.
                cp      A, #RCH_TIMEOUTDONEHIGHRES
                jr      Z, rchTargetStateTimeoutHiResDoneEvent

                ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
rchTargetStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************

                ; Enable pulse mode to change distance.
                CORE_ENABLE_PULSE_MODE
                
                ;-- If Auto-Distance active then increase distance --
                ;-- and launch the delay function --
                ld      HL,[RCHTmpStatus]
                bit     [HL],#bRCHAutoDist
                jr      Z,seL1
                    ld      A,[IY + RCHDISTOFFSET]
                    ld      B,[IY + RCHAUTODISTV]
                    add     A,B
                    ;Test if A>99
                    cp      A,#100
                    jr      C,seL2
                        ld  A,#99
seL2:               ld      [IY + RCHDISTOFFSET],A
                    
                    ; Request 5sec timeout.
                    CORE_REQ_TIMEOUT_HIRES  RCHHIRESTO_5SECONDS
seL1:                
                
                
rchTargetSubStateEntry:
                ;**************************************************************
                ;
                ;                      STOP/RESET RELEASE
                ;
                ;**************************************************************

                car rchTargetSetDisplay
                ret


;------------------------------------------------------------------------------
rchTargetStateStartSplitDepressEvent:
rchTargetStateTimeoutHiResDoneEvent:
;**************************************************************
                ;
                ;                      START/SPLIT DEPRESS
                ;
                ;**************************************************************
                ;-- Goto to the calc state --
                ld      B, #RCHCALCSTATE
                CORE_REQ_STATE_CHANGE
                ret

                
;------------------------------------------------------------------------------
rchTargetStateCWPulsesEvent:
                ;**************************************************************
                ;
                ;                      CW PULSES
                ;
                ;**************************************************************
                ; Incremente Distance
                ld      A,[IY + RCHDISTOFFSET]
                inc     A
                cp      A,#100
                jr      nz,rchCWexit
                ld      a,#01
rchCWexit:      ld      [IY + RCHDISTOFFSET],A
                car     rchTargetSetDisplay
                ret

                
;------------------------------------------------------------------------------             
rchTargetStateCCWPulsesEvent:
                ;**************************************************************
                ;
                ;                      CCW PULSES
                ;
                ;**************************************************************
                ; Decremente Distance
                ld      A,[IY + RCHDISTOFFSET]
                dec     A
                jr      nz,rchCCWexit
                ld      a,#99
rchCCWexit:     ld      [IY + RCHDISTOFFSET],A
                car     rchTargetSetDisplay
                ret

rchMsgDist:
                dw        LCDMAINDMLINE2COL1
                db        5, DM5_D, DM5_I, DM5_S, DM5_T, DM5_COLON

                
;------------------------------------------------------------------------------             
rchTargetSetDisplay:
                ;**************************************************************
                ;
                ;                      TARGET SET DISPLAY
                ;
                ;**************************************************************
                ;display the LAP icon
                LCD_UPD_LAP_FLAG ON

                ; Turn on the COLON between Segmented Digits 4 and 5
                LCD_UPD_COLON1_FLAG ON
                LCD_UPD_PER2_FLAG ON
                
                ld      IY, [CORECurrentASDAddress]

                ; Display Hour
                ld A, [IY+RCHSPLIT_H]
                ld IX, #LCDSEGDIGIT1
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ; Display Minute
                ld A, [IY+RCHSPLIT_M]
                ld IX, #LCDSEGDIGIT3
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ; Display Hour
                ld A, [IY+RCHSPLIT_S]
                ld IX, #LCDSEGDIGIT5
                LCD_DISP_2DIG_SEG_DATA_NO_ZERO_SUP
                
                ld      L, [IY + RCHDISTOFFSET]
                UTL_CONVERT_HEX_TO_2DIGIT_BCD

                ld IX, #LCDBIGCHARDMCOL23
                LCD_DISP_BIG_2DIGIT_DM_DATA_NO_ZERO_SUP
                
                ld IY, #rchMsgDist
                LCD_DISP_FORMATTED_SMALL_PROP_WIDTH_DM_MSG
                
                ld B, #05
                ld IY, #Arrow_up
                ld IX, #0F927H
                UTL_COPY_IYREG_TO_IXREG

                ld B, #05
                ld IY, #Arrow_dn
                ld IX, #0FA27H
                UTL_COPY_IYREG_TO_IXREG
                ret

;------------------------------------------------------------------------------             
Arrow_up:       db 04H,06H,1FH,06H,04H
Arrow_dn:       db 08H,18H,3EH,18H,08H

